@@ -1,3 +1,4 @@
+from functools import partial, reduce
 import json
 import shutil
 import os
@@ -5,9 +6,13 @@ import xmltodict as xml
 import requests
 from urllib.request import urlopen
 import tarfile
-from typing import Callable, List, Union
+from typing import Callable, List, Union, Dict, Any
 from .text import _get_text, _get_abstract
 
+searchtext = Union[str, List[str]]
+searchtexts = Union[searchtext, List[searchtext]]
+textlist = List[Dict[str, Any]]
+
 
 class Paperset:
     def __init__(self, directory: str) -> None:
@@ -39,7 +44,7 @@ class Paperset:
         else:
             return out
 
-    def apply(self, fn: Callable) -> list:
+    def apply(self, fn: Callable[..., Any]) -> List[Any]:
         return [fn(self._load_file(self.dir_dict[k])) for k in self.dir_dict.keys()]
 
     def texts(self) -> List[str]:
@@ -52,9 +57,10 @@ class Paperset:
         return len(self.dir_dict.keys())
 
 
-def search(ps: Paperset, txt: Union[str, List[str]]) -> List[dict]:
+def _search(ps: Union[Paperset, textlist], txt: searchtext) -> textlist:
     if type(txt) is not list:
         txt = [txt]
+    txt = [x.lower() for x in txt]
     return [
         x
         for x in ps
@@ -63,6 +69,12 @@ def search(ps: Paperset, txt: Union[str, List[str]]) -> List[dict]:
     ]
 
 
+def search(ps: Union[Paperset, textlist], terms: searchtexts) -> textlist:
+    if type(terms) is not list:
+        raise ValueError("search terms must be a list!!")
+    return reduce(lambda x, y: _search(x, y), sum([[ps], terms], []))
+
+
 def download(dir: str = ".") -> None:
     site = xml.parse(
         requests.get(
@@ -82,7 +94,7 @@ def download(dir: str = ".") -> None:
     for d in data.keys():
         print(f"downloading {data[d]}")
         handle = urlopen(data[d])
-        if d.replace(".tar.gz","") in os.listdir(f"{dir}"):
+        if d.replace(".tar.gz", "") in os.listdir(f"{dir}"):
             shutil.rmtree(f"{dir}/{d.replace('.tar.gz','')}", ignore_errors=True)
         with open(f"{dir}/{d}", "wb") as out:
             while True:
